package labs.lab09_arrays;
/**
 * Theater class for lab 9
 * 
 * @author <a href='mailto:pandre.mudry&#64;hevs.ch'> Pierre-Andr Mudry</a>
 * @version 1.0
 * 
 **/
public class Theater {
	// Stores the seats of the theater
	private Seat[][] seats;
	private final int totalRows, totalColumns;

	Theater(int rows, int columns) {
		this.totalRows = rows;
		this.totalColumns = columns;

		// Reserve space for the seats
		seats = new Seat[rows][columns];

		// Create all the seats
		char row = 'A';

		for (int i = 0; i < rows; i++) {
			for (int j = 0; j < columns; j++) {
				seats[i][j] = new Seat(row, j + 1);
				// System.out.println("Creating seat " + (j + 1) + row);
			}
			row++;
		}
	}

	/**
	 * Returns a given seat of the theater
	 */
	Seat getSeat(int row, int column) {
		return seats[row][column];
	}

	/**
	 * Returns the occupancy of a given seat
	 */
	boolean isSeatBusy(int row, int column) {
		if (row <= totalRows && row >= 0 && column <= totalColumns
				&& column >= 0) {
			return seats[row][column].busy;
		} else {
			System.out.println("Warning : You chose an invalid seat !");
			return true;
		}
	}

	/**
	 * Changes the occupancy state of a seat
	 */
	void occupySeat(int row, int column) {
		seats[row][column].busy = !seats[row][column].busy;
	}

	public String toString() {
		String result = "Theater seats occupation : \n\n";
		result += "\t";

		// First line
		for (int i = 0; i < totalColumns; i++) {
			result += i + " ";
		}

		result += "\n";

		for (int j = 0; j < totalRows; j++) {
			result += j + "\t";

			for (int i = 0; i < totalColumns; i++) {
				if (seats[j][i].busy) {
					result += "X ";
				} else {
					result += "  ";
				}
			}
			result += "\n";
		}
		return result;
	}

	/**
	 * Returns the number of occupied seats
	 */
	public int numberOfBusySeats() {
		int sum = 0;
		for (int i = 0; i < totalColumns; i++) {
			for (int j = 0; j < totalRows; j++) {
				sum += seats[j][i].busy ? 1 : 0;
			}
		}
		return sum;
	}

	/**
	 * Returns how many seats exist in total
	 */
	public int numberOfSeats() {
		return totalColumns * totalRows;
	}

	/**
	 * Display on the console how the room is occupied
	 */
	public void displayOccupation() {
		System.out.println("Theater occupation " + numberOfBusySeats() + "/"
				+ numberOfSeats());
	}

	/**
	 * Allocation algorithm for getting contiguous seats
	 * 
	 * @param nSeats
	 *            The number of contiguous seats needed
	 * @return A vector containing the seats that were allocated
	 */
	Seat[] getContinuousSeats(int nSeats) {
		Seat[] result = null;

		// Look if there is a continuous number of seats that fits
		int nSuccess = 0;
		int currentRow = 0;

		if (nSeats > totalColumns) {
			return null;
		}

		for (int j = 0; j < totalRows; j++) {
			for (int i = 0; i < totalColumns; i++) {
				if (seats[currentRow][i].busy == false) {
					nSuccess++;

					if (nSuccess == nSeats) {
						result = new Seat[nSeats];

						for (int k = 0; k < nSeats; k++) {
							result[k] = seats[currentRow][i + k - nSeats + 1];
							seats[currentRow][i + k - nSeats + 1].busy = true;
						}

						return result;
					}
				}
				else {
					nSuccess = 0;
				}
			}

			currentRow++;
			nSuccess = 0;
		}

		return result;
	}
}
